@@ -9,6 +9,7 @@
 #include "tmb_multinomPois.hpp"
 #include "tmb_distsamp.hpp"
 #include "tmb_gdistremoval.hpp"
+#include "tmb_IDS.hpp"
 #include "tmb_goccu.hpp"
 
 template<class Type>
@@ -24,10 +25,10 @@ Type objective_function<Type>::operator() () {
     return tmb_distsamp(this);
   } else if(model == "tmb_gdistremoval"){
     return tmb_gdistremoval(this);
+  } else if(model == "tmb_IDS"){
+    return tmb_IDS(this);
   } else if(model == "tmb_goccu"){
     return tmb_goccu(this);
-  } else {
-    error("Unknown model.");
   }
   return 0;
 }
